"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Service = void 0;

const tslib_1 = require("tslib");

const tapable_1 = require("tapable");

const getPaths_1 = tslib_1.__importDefault(require("./getPaths"));

const ResolvePath_1 = require("./ResolvePath");

const Prettier_1 = require("./capability/Prettier");

const Generate_1 = require("./capability/Generate");

const plugin_1 = require("./capability/plugin");

const Hooks_1 = require("./capability/Hooks");

const constants_1 = require("./constants");

const Logger_1 = require("./capability/Logger");

class Service extends ResolvePath_1.ResolvePath {
  constructor(opts) {
    var _opts$paths;

    super();
    this.userConfig = {};
    this.paths = {};
    this.configPathAliasName = '@@';
    this.methods = new Map();
    this.env = opts.env || 'development';
    this.paths = { ...opts.paths,
      ...getPaths_1.default({
        cwd: ((_opts$paths = opts.paths) === null || _opts$paths === void 0 ? void 0 : _opts$paths.cwd) || process.cwd(),
        env: this.env
      })
    };
    this.setUserConfig(opts.userConfig);
    this.prettier = new Prettier_1.PrettierFiles(this.self);
    this.generateFile = new Generate_1.GenerateFile(this.self);
    this.plugin = new plugin_1.ServerPlugin(this.self);
    this.hooks = new Hooks_1.Hooks();
    this.logger = new Logger_1.Logger();
  }

  get self() {
    return this;
  }

  setUserConfig(config) {
    if (config) {
      this.userConfig = config;
    }

    return this;
  }

  setEnv(env) {
    this.env = env;
    return this;
  }

  resolveConfigAliasNamePath(...p) {
    return [this.configPathAliasName, ...p].join('/');
  }

  forEachHooks(key, cb) {
    const hooks = this.hooks.find(key);

    for (const hook of hooks) {
      var _hook$isSkip;

      if (hook.pluginId && !this.plugin.enableStatus(hook.pluginId) || ((_hook$isSkip = hook.isSkip) === null || _hook$isSkip === void 0 ? void 0 : _hook$isSkip.call(hook))) {
        continue;
      }

      cb(hook);
    }
  }

  async applyPlugins(opts) {
    if (!opts.type || opts.type === constants_1.ApplyPluginsType.ADD) {
      const tAdd = new tapable_1.AsyncSeriesWaterfallHook(['memo']);
      this.forEachHooks(opts.key, hook => {
        tAdd.tapPromise({
          name: hook.pluginId || hook.key,
          stage: hook.stage || 0,
          before: hook.before
        }, async memo => {
          const item = await hook.fn(opts.args);
          return item ? memo.concat(item) : memo;
        });
      });
      const result = await tAdd.promise(opts.initialValue || []);
      return !result.length && opts.defaultValue ? opts.defaultValue : result;
    }

    const tEvent = new tapable_1.AsyncSeriesWaterfallHook(['_']);
    this.forEachHooks(opts.key, hook => {
      tEvent.tapPromise({
        name: hook.pluginId || hook.key,
        stage: hook.stage || 0,
        before: hook.before
      }, async (memo = []) => {
        const item = await hook.fn(opts.args);
        return item ? memo.concat([item]) : memo;
      });
    });
    return tEvent.promise(undefined);
  }

  handleReCompile() {
    this.plugin.getPlugins().forEach(plugin => {
      var _plugin$onReCompile;

      plugin === null || plugin === void 0 ? void 0 : (_plugin$onReCompile = plugin.onReCompile) === null || _plugin$onReCompile === void 0 ? void 0 : _plugin$onReCompile.call(plugin);
    });
  }

  async run(isReCompile = true) {
    if (isReCompile) {
      this.handleReCompile();
    }

    await this.applyPlugins({
      key: 'onGenerateFiles',
      type: constants_1.ApplyPluginsType.EVENT
    });
    await this.applyPlugins({
      key: 'addDepsEntryExports',
      type: constants_1.ApplyPluginsType.EVENT
    });
    await this.applyPlugins({
      key: 'onGenerateAppEntryFile',
      type: constants_1.ApplyPluginsType.EVENT
    });
    await this.prettier.setConsoleLog(false).run();
  }

}

exports.Service = Service;